#include "OutputByOrbit.h"

#include <stdio.h>

#include <fstream>
#include <iostream>
#include <string>
#include <system_error>

#include "config.h"
#include "controls.h"
#include "log.h"
#include "tools.h"

OutputByOrbitStream::OutputByOrbitStream(ctrl &ctrl, config &conf)
    : tbb::filter(serial_in_order),
      totcounts_(0),
      control_(ctrl),
      conf_(conf),
      output_file_handler_(conf_.getOutputFilenameBase(), conf_.getOutputFilenamePrefix(),
                           conf_.getNOrbitsPerFile(), conf_.getCMSSWHeaders(), conf_.getSourceID()),
      min_buffer_queue_size_(1000) {
  LOG(TRACE) << "Created output filter at " << static_cast<void *>(this);
}

void OutputByOrbitStream::OutputFixedOrbits(Slice &out) {
  uint32_t orbitN = out.get_firstOrbitN();
  uint32_t index = uint32_t(orbitN / conf_.getNOrbitsPerFile());
  size_t n = 0;

  if ((out.get_counts() != 0) || conf_.getCMSSWHeaders()) {
    if (control_.running.load(std::memory_order_acquire) ||
        control_.output_force_write) {  // i.e should be writing data
      if (!out.isInitialized()) {
        return;
      }
      n = fwrite(out.begin(), 1, out.size(),
                 output_file_handler_.getFile(control_.run_number, index).getFilePtr());
      output_file_handler_.upFileSize(n);
      output_file_handler_.upNOrbits(conf_.getNOrbitsPerPacket());
    } else if (output_file_handler_.hasFile()) {
      // the run has been stopped so drop but first check if there is a last
      // file to close
      LOG(TRACE) << "the run was stopped. queueing the last file for close and "
                    "rename ";
      output_file_handler_.enqueue_current_file_for_close_and_move_maybe();
      // TODO: Need to write EoR file here, probably.
    }
  }
}
void *OutputByOrbitStream::operator()(void *item) {
  Slice &out = *static_cast<Slice *>(item);
  totcounts_ += out.get_counts();
  OutputFixedOrbits(out);
  if (Slice::current_queue_size() < min_buffer_queue_size_) {
    LOG(TRACE) << "New minimum queue size is " << Slice::current_queue_size() << " total counts "
               << totcounts_;
    min_buffer_queue_size_ = Slice::current_queue_size();
  }
  Slice::giveAllocated(&out);
  return NULL;
}
