#include "config.h"

#include <fstream>

#include "log.h"

config::config(std::string filename) {
  bool valid = true;

  std::ifstream in(filename.c_str(), std::ios_base::in);

  if (!in.is_open()) {
    throw std::invalid_argument("Configuration file could not be opened. Does it exist?");
  }

  std::string line;
  while (!std::getline(in, line).eof()) {
    // Perform left trim
    line.erase(line.begin(),
               std::find_if(line.begin(), line.end(), [](int ch) { return !std::isspace(ch); }));

    // Skip empty lines
    if (line.empty()) {
      continue;
    }

    // Skip comments
    if (line.size() > 1 && line[0] == '#') {
      continue;
    }

    auto delim = line.find_first_of(":");
    if (delim == std::string::npos) {
      // Skip ill formated lines
      continue;
    }

    std::string key = line.substr(0, delim);
    std::string value = line.substr(delim + 1);

    if (key.empty()) {
      // Skip ill formated lines
      continue;
    }

    if (value.empty()) {
      LOG(ERROR) << "Configuration entry " << key << " has no value!";
      valid &= false;
    }

    vmap[key] = value;
  }

  if (!valid) {
    throw std::invalid_argument("Configuration error: Keys with no value detected!");
  }
}
void config::print() const {
  for (std::map<std::string, std::string>::const_iterator it = vmap.begin(); it != vmap.end();
       it++) {
    LOG(INFO) << "key " << it->first << " value " << it->second;
  }
}

uint32_t config::getSourceID() {
  uint32_t source_id = 0;
  if (getProcessorType() == StreamProcessor::ProcessorType::GMT) {
    source_id = 1;
  } else if (getProcessorType() == StreamProcessor::ProcessorType::CALO) {
    source_id = 2;
  } else {
    if (getCMSSWHeaders() == true) {
      LOG(ERROR) << "PROCESSOR_TYPE INCOMPATIBLE WITH CMSSW HEADER OUTPUT OPTION, PLEASE DISABLE "
                    "CMSSW HEADERS OR CHANGE PROCESSOR_TYPE, EXITING";
      throw std::invalid_argument(
          "ERROR: PROCESSOR_TYPE INCOMPATIBLE WITH CMSSW HEADER OUTPUT OPTION, PLEASE DISABLE "
          "CMSSW HEADERS OR CHANGE PROCESSOR_TYPE");
    } else {
      LOG(WARNING) << "Undefined source ID has been requested: returning 0.";
    }
  }
  return source_id;
}
