#ifndef PIPELINE_H
#define PIPELINE_H

#include <sysexits.h>

#include <boost/asio.hpp>
#include <boost/bind.hpp>
#include <boost/thread.hpp>
#include <cctype>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <iostream>
#include <memory>

#include "DmaInputFilter.h"
#include "FileDmaInputFilter.h"
#include "InputFilter.h"
#include "MicronDmaInputFilter.h"
#include "OutputByOrbit.h"
#include "OutputBySize.h"
#include "TcpInputFilter.h"
#include "WZDmaInputFilter.h"
#include "config.h"
#include "controls.h"
#include "format.h"
#include "processor.h"
#include "server.h"
#include "tbb/concurrent_queue.h"
#include "tbb/pipeline.h"
#include "tbb/task_group.h"
#include "tbb/task_scheduler_init.h"
#include "tbb/tbb_allocator.h"
#include "tbb/tick_count.h"
#include "tools.h"

class Pipeline {
  friend class MultiPipeline;
  using OutputFilter = tbb::filter;

 public:
  static ctrl control;
  // Impacts the maximum number of in-flight TBB tokens
  const uint fMaxTokensPerThread;
  config fConf;

  std::shared_ptr<InputFilter> fInputFilter;
  StreamProcessor fStreamProcessor;
  std::shared_ptr<OutputFilter> fOutputStream;
  tbb::pipeline fPipeline;

  explicit Pipeline(config &conf, uint maxTokensPerThread = 4);
};

class MultiPipeline {
 public:
  std::deque<Pipeline> fPipelines;

  tbb::task_scheduler_init fSchedulerInit;
  boost::asio::io_service io_service;
  boost::thread fServerThread;
  std::unique_ptr<server> fServer;
  bool fVerbose = true;

  explicit MultiPipeline(std::vector<config> &configs);
  int Run();

 private:
  int RunPipelines(uint numStreams);
};

#endif